#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from sql_to_python import QuerySQL
from sql_connector import DVH_SQL
from datetime import datetime
from dateutil.relativedelta import relativedelta
from dicompylercore import dicomparser
import os
import dicom
import sys


class Temp_DICOM_FileSet:
    def __init__(self, **kwargs):

        # Read SQL configuration file
        script_dir = os.path.dirname(__file__)
        if 'start_path' in kwargs:
            rel_path = kwargs['start_path']
            abs_file_path = os.path.join(script_dir, rel_path)
            start_path = abs_file_path
        else:
            rel_path = "preferences/import_settings.txt"
            abs_file_path = os.path.join(script_dir, rel_path)
            with open(abs_file_path, 'r') as document:
                for line in document:
                    line = line.split()
                    if not line:
                        continue
                    if line[0] == 'review':
                        start_path = line[1:][0]

        self.plan = []
        self.structure = []
        self.dose = []
        self.mrn = []
        self.study_instance_uid = []

        f = []

        for root, dirs, files in os.walk(start_path, topdown=False):
            for name in files:
                f.append(os.path.join(root, name))

        plan_files = []
        study_uid_plan = []
        structure_files = []
        study_uid_structure = []
        dose_files = []
        study_uid_dose = []
        mrns = []

        for x in range(0, len(f)):
            try:
                dicom_file = dicom.read_file(f[x])
                if dicom_file.Modality.lower() == 'rtplan':
                    plan_files.append(f[x])
                    study_uid_plan.append(dicom_file.StudyInstanceUID)
                elif dicom_file.Modality.lower() == 'rtstruct':
                    structure_files.append(f[x])
                    study_uid_structure.append(dicom_file.StudyInstanceUID)
                elif dicom_file.Modality.lower() == 'rtdose':
                    dose_files.append(f[x])
                    study_uid_dose.append(dicom_file.StudyInstanceUID)
            except Exception:
                pass

        self.count = len(plan_files)

        for a in range(0, self.count):
            self.plan.append(plan_files[a])
            self.mrn.append(dicom.read_file(plan_files[a]).PatientID)
            self.study_instance_uid.append(dicom.read_file(plan_files[a]).StudyInstanceUID)
            for b in range(0, len(structure_files)):
                if study_uid_plan[a] == study_uid_structure[b]:
                    self.structure.append(structure_files[b])
            for c in range(0, len(dose_files)):
                if study_uid_plan[a] == study_uid_dose[c]:
                    self.dose.append(dose_files[c])

        if self.count == 0:
            self.plan.append('')
            self.mrn.append('')
            self.structure.append('')
            self.dose.append('')

    def get_roi_names(self, mrn):

        structure_file = self.structure[self.mrn.index(mrn)]
        rt_st = dicomparser.DicomParser(structure_file)
        rt_structures = rt_st.GetStructures()

        roi = {}
        for key in rt_structures:
            if rt_structures[key]['type'].upper() not in {'MARKER', 'REGISTRATION', 'ISOCENTER'}:
                roi[key] = rt_structures[key]['name']

        return roi


def recalculate_ages():

    dvh_data = QuerySQL('Plans', "mrn != ''")
    cnx = DVH_SQL()

    for i in range(0, len(dvh_data.mrn)):
        uid = dvh_data.study_instance_uid[i]
        sim_study_date = dvh_data.sim_study_date[i].split('-')
        birth_date = dvh_data.birth_date[i].split('-')

        birth_year = int(birth_date[0])
        birth_month = int(birth_date[1])
        birth_day = int(birth_date[2])
        birth_date_obj = datetime(birth_year, birth_month, birth_day)

        sim_study_year = int(sim_study_date[0])
        sim_study_month = int(sim_study_date[1])
        sim_study_day = int(sim_study_date[2])
        sim_study_date_obj = datetime(sim_study_year,
                                               sim_study_month,
                                               sim_study_day)

        if sim_study_date == '1800-01-01':
            age = '(NULL)'
        else:
            age = relativedelta(sim_study_date_obj, birth_date_obj).years

        condition = "study_instance_uid = '" + uid + "'"
        cnx.update('Plans', 'age', str(age), condition)

    cnx.cnx.close()


def datetime_str_to_obj(datetime_str):

    year = int(datetime_str[0:4])
    month = int(datetime_str[4:6])
    day = int(datetime_str[6:8])
    hour = int(datetime_str[8:10])
    minute = int(datetime_str[10:12])
    second = int(datetime_str[12:14])

    datetime_obj = datetime(year, month, day, hour, minute, second)

    return datetime_obj


def date_str_to_obj(date_str):

    year = int(date_str[0:4])
    month = int(date_str[4:6])
    day = int(date_str[6:8])

    return datetime(year, month, day)


def platform():
    if sys.platform.startswith('win'):
        return 'windows'
    elif sys.platform.startswith('darwin'):
        return 'mac'
    return 'linux'



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu May 23 16:57 2017
@author: Dan Cutright, PhD
This is the main python file for command line implementation.
"""

from dicom_to_sql import dicom_to_sql
from utilities import recalculate_ages, Temp_DICOM_FileSet
from sql_connector import DVH_SQL
from analysis_tools import DVH
import os
from getpass import getpass
import argparse
from subprocess import call


def is_import_settings_defined():

    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/import_settings.txt"
    abs_file_path = os.path.join(script_dir, rel_path)

    if os.path.isfile(abs_file_path):
        return True
    else:
        return False


def is_sql_connection_defined():

    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/sql_connection.cnf"
    abs_file_path = os.path.join(script_dir, rel_path)

    if os.path.isfile(abs_file_path):
        return True
    else:
        return False


def validate_import_settings():
    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/import_settings.txt"
    abs_file_path = os.path.join(script_dir, rel_path)

    with open(abs_file_path, 'r') as document:
        config = {}
        for line in document:
            line = line.split()
            if not line:
                continue
            config[line[0]] = line[1:][0]

    valid = True
    for key, value in config.iteritems():
        if not os.path.isdir(value):
            print 'invalid', key, 'path: ', value
            valid = False

    return valid


def validate_sql_connection():

    try:
        cnx = DVH_SQL()
        cnx.close()
        valid = True
    except:
        valid = False

    if valid:
        print "SQL DB is alive!"
    else:
        print "Connection to SQL DB could not be established."
        if not is_sql_connection_defined():
            print "ERROR: SQL settings are not yet defined.  Please run:\n    $ python command_line.py" \
                  " settings --sql"

    return valid


def settings(**kwargs):
    if not kwargs:
        set_import_settings()
        set_sql_connection_parameters()
    else:
        if 'dir' in kwargs and kwargs['dir']:
            set_import_settings()
        if 'sql' in kwargs and kwargs['sql']:
            set_sql_connection_parameters()


def test_dvh_code():

    if not is_import_settings_defined() and not is_sql_connection_defined():
        print "ERROR: Import and SQL settings are not yet defined.  Please run:\n    $ python command_line.py" \
              " settings"
    elif not is_import_settings_defined():
        print "ERROR: Import settings are not yet defined.  Please run:\n    $ python command_line.py" \
              " settings --dir"
    elif not is_sql_connection_defined():
        print "ERROR: SQL settings are not yet defined.  Please run:\n    $ python command_line.py" \
              " settings --sql"
    else:
        is_import_valid = validate_import_settings()
        is_sql_connection_valid = validate_sql_connection()
        if not is_import_valid and not is_sql_connection_valid:
            print("ERROR: Create the directories listed above or input valid directories.")
            print("ERROR: Cannot connect to SQL.")
            print("Please run:")
            print("    $ python command_line.py"
                  " settings")
        elif not is_import_valid:
            print("ERROR: Create the directories listed above or input valid directories by running:")
            print("    $ python command_line.py"
                  " settings --dir")
        elif not is_sql_connection_valid:
            print("ERROR: Cannot connect to SQL.")
            print("Verify database is active and/or update SQL connection information with:")
            print("    $ python command_line.py"
                  " settings --sql")

        else:
            print "Importing test files"
            dicom_to_sql(start_path="test_files/",
                         organize_files=False,
                         move_files=False,
                         force_update=False)

            print "Reading data from SQL DB with analysis_tools.py"
            test = DVH()

            print "Reading dicom information from test files with utilities.py (for plan review module)"
            test_files = Temp_DICOM_FileSet(start_path="test_files/")

            print "Deleting test data from SQL database"
            for i in range(0, test_files.count):
                cond_str = "mrn = '" + test_files.mrn[i]
                cond_str += "' and study_instance_uid = '" + test_files.study_instance_uid[i] + "'"
                DVH_SQL().delete_rows(cond_str)

            print "Tests successful!"


def get_import_settings_from_user():
    print "Please enter the full directory path for each category"

    print "\nThis is where dicom files live before import."
    inbox_file_path = raw_input('Inbox: ')

    print "\nThis is where dicom files move to after import."
    imported_file_path = raw_input('Imported: ')

    print "\nThis is where dicom files to be reviewed live, but will not be imported."
    review_file_path = raw_input('DVH Review: ')

    import_settings = {'inbox': str(inbox_file_path),
                       'imported': str(imported_file_path),
                       'review': str(review_file_path)}

    return import_settings


def get_sql_connection_parameters_from_user():

    print "\nPlease enter the host address\n(defaults to 'localhost' if left empty)"
    host = raw_input('Host: ')
    if not host:
        host = 'localhost'

    print "\nPlease enter the user name\n(leave empty for OS authentication)"
    user = raw_input('User: ')

    if user:
        print "\nPlease enter the password, if any\n(will not display key strokes)"
        password = getpass('Password: ')

    print "\nPlease enter the database name\n(defaults to dvh if empty)"
    dbname = raw_input('Database name: ')
    if not dbname:
        dbname = 'dvh'

    print "\nPlease enter the database port\n(defaults to PostgreSQL default: 5432)"
    port = raw_input('Port: ')
    if not port:
        port = '5432'

    sql_connection_parameters = {'host': str(host),
                                 'dbname': str(dbname),
                                 'port': str(port)}

    if user:
        sql_connection_parameters['user'] = str(user)
        sql_connection_parameters['password'] = str(password)

    return sql_connection_parameters


def write_import_settings(settings):

    import_text = ['inbox ' + settings['inbox'],
                   'imported ' + settings['imported'],
                   'review ' + settings['review']]
    import_text = '\n'.join(import_text)

    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/import_settings.txt"
    abs_file_path = os.path.join(script_dir, rel_path)

    with open(abs_file_path, "w") as text_file:
        text_file.write(import_text)


def write_sql_connection_settings(config):

    text = []
    for key, value in config.iteritems():
        text.append(key + ' ' + value)
    text = '\n'.join(text)

    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/sql_connection.cnf"
    abs_file_path = os.path.join(script_dir, rel_path)

    with open(abs_file_path, "w") as text_file:
        text_file.write(text)


def set_import_settings():
    config = get_import_settings_from_user()
    write_import_settings(config)


def set_sql_connection_parameters():
    config = get_sql_connection_parameters_from_user()
    write_sql_connection_settings(config)


def print_mrns():
    mrns = DVH_SQL().get_unique_values('plans', 'mrn')
    if len(mrns) == 0:
        print "No plans have been imported"

    printed_mrns = []
    for i in range(0, len(mrns)):
        current_mrn = mrns[i]
        if current_mrn not in printed_mrns:
            printed_mrns.append(current_mrn)
            print current_mrn


def main():
    parser = argparse.ArgumentParser(description='Command line interface for DVH Analytics')
    parser.add_argument('--sql',
                        help='Modify SQL connection settings',
                        default=False,
                        action='store_true')
    parser.add_argument('--dir',
                        help='Modify import directory settings',
                        default=False,
                        action='store_true')
    parser.add_argument('--start-path',
                        dest='start_path',
                        help='modify the start path for dicom import',
                        default=None)
    parser.add_argument('--do-not-organize',
                        help='Import will organize files by default. Set to False to leave files in the same hierarchy',
                        default=False,
                        action='store_true')
    parser.add_argument('--do-not-move',
                        help='Import will move files by default.  Set to False to leave files where they are',
                        default=False,
                        action='store_true')
    parser.add_argument('--force-update',
                        help='Import will add to SQL DB even if DB already contains data from dicom files',
                        default=False,
                        action='store_true')
    parser.add_argument('command', nargs='+', help='bar help')
    args = parser.parse_args()

    if args.command:

        if args.command[0] == 'settings':
            if not args.sql and not args.dir:
                settings()
            else:
                if args.sql:
                    settings(sql=True)
                if args.dir:
                    settings(dir=True)

        elif args.command[0] == 'test':
            test_dvh_code()

        elif args.command[0] == 'echo':
            validate_sql_connection()

        elif args.command[0] == 'print_mrns':
            print_mrns()

        elif args.command[0] == 'recalculate_ages':
            recalculate_ages()

        elif args.command[0] == 'import':

            # Set defaults
            start_path = False
            organize_files = True
            move_files = True
            force_update = False

            if args.start_path:
                if os.path.isdir(args.start_path):
                    start_path = str(args.start_path)
                else:
                    print args.start_path, 'is not a valid path'
            if args.do_not_organize:
                organize_files = False
            if args.do_not_move:
                move_files = False
            if args.force_update:
                force_update = True

            dicom_to_sql(start_path=start_path,
                         organize_files=organize_files,
                         move_files=move_files,
                         force_update=force_update)
        elif args.command[0] == 'run':
            script_dir = os.path.dirname(__file__)

            call(["bokeh", "serve", "--show", script_dir])


if __name__ == '__main__':
    main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# DICOM_to_Python.py
"""
Import DICOM RT Dose, Structure, and Plan files into Python objects
Created on Sun Feb 26 11:06:28 2017
@author: Dan Cutright, PhD
"""

import dicom  # pydicom
from dicompylercore import dicomparser, dvhcalc
from datetime import datetime
from dateutil.relativedelta import relativedelta  # python-dateutil
from roi_name_manager import DatabaseROIs, clean_name
from utilities import datetime_str_to_obj


# Each ROI class object contains data to fill an entire row of the SQL table 'DVHs'
# This code will generate a list of ROI class objects
# There will be a ROI class per structure of a Plan
class DVHRow:
    def __init__(self, mrn, study_instance_uid, institutional_roi, physician_roi,
                 roi_name, roi_type, volume, min_dose, mean_dose, max_dose, dvh_str):
        self.mrn = mrn
        self.study_instance_uid = study_instance_uid
        self.institutional_roi = institutional_roi
        self.physician_roi = physician_roi
        self.roi_name = roi_name
        self.roi_type = roi_type
        self.volume = volume
        self.min_dose = min_dose
        self.mean_dose = mean_dose
        self.max_dose = max_dose
        self.dvh_str = dvh_str


# Each BeamInfo class object contains data to fill an entire row of the SQL table 'Beams'
# This code will generate a list of BeamInfo class objects
# There will be a Beam class per beam of a Plan
class BeamRow:
    def __init__(self, mrn, study_instance_uid, beam_number, beam_name,
                 fx_group, fxs, fx_grp_beam_count, beam_dose,
                 beam_mu, radiation_type, beam_energy, beam_type, control_point_count,
                 gantry_start, gantry_end, gantry_rot_dir, collimator_angle,
                 couch_angle, isocenter, ssd, treatment_machine):
        self.mrn = mrn
        self.study_instance_uid = study_instance_uid
        self.beam_number = beam_number
        self.beam_name = beam_name
        self.fx_group = fx_group
        self.fxs = fxs
        self.fx_grp_beam_count = fx_grp_beam_count
        self.beam_dose = beam_dose
        self.beam_mu = beam_mu
        self.radiation_type = radiation_type
        self.beam_energy = beam_energy
        self.beam_type = beam_type
        self.control_point_count = control_point_count
        self.gantry_start = gantry_start
        self.gantry_end = gantry_end
        self.gantry_rot_dir = gantry_rot_dir
        self.collimator_angle = collimator_angle
        self.couch_angle = couch_angle
        self.isocenter = isocenter
        self.ssd = ssd
        self.treatment_machine = treatment_machine


class RxRow:
    def __init__(self, mrn, study_instance_uid, plan_name, fx_grp_name, fx_grp_number, fx_grp_count,
                 fx_dose, fxs, rx_dose, rx_percent, normalization_method, normalization_object):
        self.mrn = mrn
        self.study_instance_uid = study_instance_uid
        self.plan_name = plan_name
        self.fx_grp_name = fx_grp_name
        self.fx_grp_number = fx_grp_number
        self.fx_grp_count = fx_grp_count
        self.fx_dose = fx_dose
        self.fxs = fxs
        self.rx_dose = rx_dose
        self.rx_percent = rx_percent
        self.normalization_method = normalization_method
        self.normalization_object = normalization_object


# Each Plan class object contains data to fill an entire row of the SQL table 'Plans'
# There will be a Plan class per structure of a Plan
class PlanRow:
    def __init__(self, plan_file, structure_file, dose_file):
        # Read DICOM files
        rt_plan = dicom.read_file(plan_file)
        rt_plan_dicompyler = dicomparser.DicomParser(plan_file)
        rt_plan_obj = rt_plan_dicompyler.GetPlan()
        rt_structure = dicom.read_file(structure_file)
        rt_dose = dicom.read_file(dose_file)

        if isinstance(rt_dose.TissueHeterogeneityCorrection, basestring):
            heterogeneity_correction = rt_dose.TissueHeterogeneityCorrection
        else:
            heterogeneity_correction = ','.join(rt_dose.TissueHeterogeneityCorrection)

        # Record Medical Record Number
        MRN = rt_plan.PatientID

        # Record gender
        patient_sex = rt_plan.PatientSex.upper()
        if not (patient_sex == 'M' or patient_sex == 'F'):
            patient_sex = '-'

        # Parse and record sim date
        sim_study_date = rt_plan.StudyDate
        if sim_study_date:
            sim_study_year = int(sim_study_date[0:4])
            sim_study_month = int(sim_study_date[4:6])
            sim_study_day = int(sim_study_date[6:8])
            sim_study_date_obj = datetime(sim_study_year, sim_study_month, sim_study_day)
        else:
            sim_study_date = '(NULL)'

        # Calculate patient age at time of sim
        # Set to NULL birthday is not in DICOM file
        birth_date = rt_plan.PatientBirthDate
        if not birth_date:
            birth_date = '(NULL)'
            age = '(NULL)'
        else:
            birth_year = int(birth_date[0:4])
            birth_month = int(birth_date[4:6])
            birth_day = int(birth_date[6:8])
            birth_date_obj = datetime(birth_year, birth_month, birth_day)
            if sim_study_date == '(NULL)':
                age = '(NULL)'
            else:
                age = relativedelta(sim_study_date_obj, birth_date_obj).years

        # Record physician initials
        # In Pinnacle, PhysiciansOfRecord refers to the Radiation Oncologist field
        if hasattr(rt_plan, 'PhysiciansOfRecord'):
            physician = rt_plan.PhysiciansOfRecord.upper()
        else:
            physician = rt_plan.ReferringPhysicianName.upper()

        # Initialize fx and MU counters, iterate over all rx's
        fxs = 0
        total_mu = 0
        fx_grp_seq = rt_plan.FractionGroupSequence  # just to limit characters for later reference
        # Count number of fxs and total MUs
        # Note these are total fxs, not treatment days
        for fxgrp in range(0, len(fx_grp_seq)):
            fxs += fx_grp_seq[fxgrp].NumberOfFractionsPlanned
            for Beam in range(0, fx_grp_seq[fxgrp].NumberOfBeams):
                total_mu += fx_grp_seq[fxgrp].ReferencedBeamSequence[Beam].BeamMeterset

        # This UID must be in all DICOM files associated with this sim study
        study_instance_uid = rt_plan.StudyInstanceUID

        # Record patient position (e.g., HFS, HFP, FFS, or FFP)
        patient_orientation = rt_plan.PatientSetupSequence[0].PatientPosition

        # Context self-evident, their utility is not (yet)
        plan_time_stamp = datetime_str_to_obj(rt_plan.RTPlanDate + rt_plan.RTPlanTime)
        struct_time_stamp = datetime_str_to_obj(rt_structure.StructureSetDate + rt_structure.StructureSetTime)
        dose_time_stamp = datetime_str_to_obj(rt_dose.ContentDate + rt_dose.ContentTime)

        # Record treatment planning system vendor, name, and version
        tps_manufacturer = rt_plan.Manufacturer
        tps_software_name = rt_plan.ManufacturerModelName
        tps_software_version = rt_plan.SoftwareVersions[0]

        # Because DICOM does not contain Rx's explicitly, the user must create
        # a point in the RT Structure file called 'rx: '
        # If multiple Rx found, sum will be reported
        # Record tx site from 'tx:' point or RTPlanLabel if no point
        tx_site = rt_plan.RTPlanLabel  # tx_site defaults to plan name if tx in a tx point
        tx_found = False
        rx_dose = 0
        roi_counter = 0
        roi_count = len(rt_structure.StructureSetROISequence)
        roi_seq = rt_structure.StructureSetROISequence  # just limit num characters in code
        fx_reset = False
        while (not tx_found) and (roi_counter < roi_count):
            roi_name = roi_seq[roi_counter].ROIName.lower()
            if len(roi_name) > 2:
                temp = roi_name.split(' ')
                if temp[0][0:2] == 'rx':
                    if not fx_reset:
                        fxs = 0
                        fx_reset = True
                    fx_dose = float(temp[temp.index('cgy')-1]) / 100
                    fxs += int(temp[temp.index('x') + 1])
                    rx_dose += fx_dose * float((temp[temp.index('x') + 1]))
                elif temp[0] == 'tx:':
                    temp.remove('tx:')
                    tx_site = ' '.join(temp)
                    tx_found = True
            roi_counter += 1

        # if rx_dose point not found, use dose in DICOM file
        if rx_dose == 0:
            rx_dose = float(rt_plan_obj['rxdose']) / 100

        # This assumes that Plans are either 100% Arc plans or 100% Static Angle
        # Note that the beams class will have this information on a per beam basis
        tx_modality = ''
        tx_energies = ' '
        temp = ''
        if rt_plan_obj['brachy']:
            tx_modality = 'Brachy '
        else:
            beam_seq = rt_plan.BeamSequence
            for beam_num in range(0, len(beam_seq) - 1):
                temp += beam_seq[beam_num].RadiationType + ' '
                first_cp = beam_seq[beam_num].ControlPointSequence[0]
                if first_cp.GantryRotationDirection in {'CW', 'CC'}:
                    temp += 'Arc '
                else:
                    temp += '3D '
                energy_temp = ' ' + str(first_cp.NominalBeamEnergy)
                if beam_seq[beam_num].RadiationType.lower() == 'photon':
                    energy_temp += 'MV '
                elif beam_seq[beam_num].RadiationType.lower() == 'proton':
                    energy_temp += 'MeV '
                elif beam_seq[beam_num].RadiationType.lower() == 'electron':
                    energy_temp += 'MeV '
                if tx_energies.find(energy_temp) < 0:
                    tx_energies += energy_temp
            tx_energies = tx_energies[1:len(tx_energies) - 1]
            if temp.lower().find('photon') > -1:
                if temp.lower().find('photon arc') > -1:
                    tx_modality += 'Photon Arc '
                else:
                    tx_modality += 'Photon 3D '
            if temp.lower().find('electron') > -1:
                if temp.lower().find('electron arc') > -1:
                    tx_modality += 'Electron Arc '
                else:
                    tx_modality += 'Electron 3D '
            elif temp.lower().find('proton') > -1:
                tx_modality += 'Proton '
        tx_modality = tx_modality[0:len(tx_modality) - 1]

        # Will require a yet to be named function to determine this
        # Applicable for brachy and Gamma Knife, not yet supported
        tx_time = '00:00:00'

        # Record resolution of dose grid
        dose_grid_resolution = [str(round(float(rt_dose.PixelSpacing[0]), 1)),
                                str(round(float(rt_dose.PixelSpacing[1]), 1)),
                                str(round(float(rt_dose.SliceThickness), 1))]
        dose_grid_resolution = ', '.join(dose_grid_resolution)

        # Set object values
        self.mrn = MRN
        self.birth_date = birth_date
        self.age = age
        self.patient_sex = patient_sex
        self.sim_study_date = sim_study_date
        self.physician = physician
        self.tx_site = tx_site
        self.rx_dose = rx_dose
        self.fxs = fxs
        self.tx_energies = tx_energies
        self.study_instance_uid = study_instance_uid
        self.patient_orientation = patient_orientation
        self.plan_time_stamp = plan_time_stamp
        self.struct_time_stamp = struct_time_stamp
        self.dose_time_stamp = dose_time_stamp
        self.tps_manufacturer = tps_manufacturer
        self.tps_software_name = tps_software_name
        self.tps_software_version = tps_software_version
        self.tx_modality = tx_modality
        self.tx_time = tx_time
        self.total_mu = total_mu
        self.dose_grid_resolution = dose_grid_resolution
        self.heterogeneity_correction = heterogeneity_correction


class DVHTable:
    def __init__(self, structure_file, dose_file):
        # Get ROI Category Map
        database_rois = DatabaseROIs()

        # Import RT Structure and RT Dose files using dicompyler
        rt_structure_dicom = dicom.read_file(structure_file)
        mrn = rt_structure_dicom.PatientID
        study_instance_uid = rt_structure_dicom.StudyInstanceUID

        rt_structure = dicomparser.DicomParser(structure_file)
        rt_structures = rt_structure.GetStructures()

        if hasattr(rt_structure_dicom, 'PhysiciansOfRecord'):
            physician = rt_structure_dicom.PhysiciansOfRecord.upper()
        else:
            physician = rt_structure_dicom.ReferringPhysicianName.upper()

        values = {}
        row_counter = 0
        self.dvhs = {}
        for key in rt_structures:
            # Import DVH from RT Structure and RT Dose files
            if rt_structures[key]['type'] != 'MARKER':

                current_dvh_calc = dvhcalc.get_dvh(structure_file, dose_file, key)
                self.dvhs[row_counter] = current_dvh_calc.counts
                if current_dvh_calc.volume > 0:
                    print 'Importing ' + current_dvh_calc.name
                    if rt_structures[key]['name'].lower().find('itv') == 0:
                        roi_type = 'ITV'
                    else:
                        roi_type = rt_structures[key]['type']
                    current_roi_name = clean_name(rt_structures[key]['name'])

                    if database_rois.is_roi(current_roi_name):
                        if database_rois.is_physician(physician):
                            institutional_roi = database_rois.get_institutional_roi(physician, current_roi_name)
                            physician_roi = database_rois.get_physician_roi(physician, current_roi_name)
                        else:
                            if current_roi_name in database_rois.institutional_rois:
                                institutional_roi = current_roi_name
                            else:
                                institutional_roi = 'uncategorized'
                            physician_roi = 'uncategorized'
                    else:
                        institutional_roi = 'uncategorized'
                        physician_roi = 'uncategorized'

                    current_dvh_row = DVHRow(mrn,
                                             study_instance_uid,
                                             institutional_roi,
                                             physician_roi,
                                             current_roi_name,
                                             roi_type,
                                             current_dvh_calc.volume,
                                             current_dvh_calc.min,
                                             current_dvh_calc.mean,
                                             current_dvh_calc.max,
                                             ','.join(['%.2f' % num for num in current_dvh_calc.counts]))
                    values[row_counter] = current_dvh_row
                    row_counter += 1

        self.count = row_counter
        dvh_range = range(0, self.count)

        self.mrn = [values[x].mrn for x in dvh_range]
        self.study_instance_uid = [values[x].study_instance_uid for x in dvh_range]
        self.institutional_roi = [values[x].institutional_roi for x in dvh_range]
        self.physician_roi = [values[x].physician_roi for x in dvh_range]
        self.roi_name = [values[x].roi_name for x in dvh_range]
        self.roi_type = [values[x].roi_type for x in dvh_range]
        self.volume = [values[x].volume for x in dvh_range]
        self.min_dose = [values[x].min_dose for x in dvh_range]
        self.mean_dose = [values[x].mean_dose for x in dvh_range]
        self.max_dose = [values[x].max_dose for x in dvh_range]
        self.dvh_str = [values[x].dvh_str for x in dvh_range]


class BeamTable:
    def __init__(self, plan_file):

        beam_num = 0
        values = {}
        # Import RT Dose files using dicompyler
        rt_plan = dicom.read_file(plan_file)

        mrn = rt_plan.PatientID
        study_instance_uid = rt_plan.StudyInstanceUID

        for fx_grp in range(0, len(rt_plan.FractionGroupSequence)):
            fx_grp_seq = rt_plan.FractionGroupSequence[fx_grp]
            fxs = int(fx_grp_seq.NumberOfFractionsPlanned)
            fx_grp_beam_count = int(fx_grp_seq.NumberOfBeams)

            for fx_grp_beam in range(0, fx_grp_beam_count):
                beam_seq = rt_plan.BeamSequence[beam_num]
                if 'BeamDescription' in beam_seq:
                    beam_name = beam_seq.BeamDescription
                else:
                    beam_name = beam_seq.BeamName
                ref_beam_seq = fx_grp_seq.ReferencedBeamSequence[fx_grp_beam]
                treatment_machine = beam_seq.TreatmentMachineName

                beam_dose = float(ref_beam_seq.BeamDose)
                beam_mu = float(ref_beam_seq.BeamMeterset)

                isocenter = [str(round(ref_beam_seq.BeamDoseSpecificationPoint[0], 4)),
                             str(round(ref_beam_seq.BeamDoseSpecificationPoint[1], 4)),
                             str(round(ref_beam_seq.BeamDoseSpecificationPoint[2], 4))]
                isocenter = ','.join(isocenter)

                radiation_type = beam_seq.RadiationType
                beam_energy = beam_seq.ControlPointSequence[0].NominalBeamEnergy
                beam_type = beam_seq.BeamType

                control_point_count = beam_seq.NumberOfControlPoints
                first_cp = beam_seq.ControlPointSequence[0]
                if beam_seq.BeamType == 'STATIC':
                    final_cp = first_cp
                else:
                    final_cp = beam_seq.ControlPointSequence[control_point_count - 1]
                gantry_start = float(first_cp.GantryAngle)
                gantry_rot_dir = first_cp.GantryRotationDirection
                gantry_end = float(final_cp.GantryAngle)
                collimator_angle = float(first_cp.BeamLimitingDeviceAngle)
                if collimator_angle > 180:
                    collimator_angle -= 360
                couch_angle = float(first_cp.PatientSupportAngle)
                if couch_angle > 180:
                    couch_angle -= 360

                # If beam is an arc, return average SSD, otherwise
                if gantry_rot_dir in {'CW', 'CC'}:
                    ssd = 0
                    for CP in range(0, control_point_count):
                        ssd += round(float(beam_seq.ControlPointSequence[CP].SourceToSurfaceDistance) / 10, 2)
                    ssd /= control_point_count
                    if gantry_rot_dir == 'CW':
                        if gantry_start > 176:
                            gantry_start -= float(360)
                        if gantry_end > 180:
                            gantry_end -= float(360)
                    elif gantry_rot_dir == 'CC':
                        if gantry_start > 184:
                            gantry_start -= float(360)
                        if gantry_end > 180:
                            gantry_end -= float(360)
                else:
                    gantry_rot_dir = '-'
                    ssd = float(first_cp.SourceToSurfaceDistance) / 10

                current_beam = BeamRow(mrn, study_instance_uid, beam_num + 1,
                                       beam_name, fx_grp + 1, fxs,
                                       fx_grp_beam_count, beam_dose, beam_mu, radiation_type,
                                       beam_energy, beam_type, control_point_count,
                                       gantry_start, gantry_end, gantry_rot_dir,
                                       collimator_angle, couch_angle, isocenter, ssd, treatment_machine)

                values[beam_num] = current_beam
                beam_num += 1

        self.count = beam_num
        beam_range = range(0, self.count)

        # Rearrange values into separate variables
        self.mrn = [values[x].mrn for x in beam_range]
        self.study_instance_uid = [values[x].study_instance_uid for x in beam_range]
        self.beam_number = [values[x].beam_number for x in beam_range]
        self.beam_name = [values[x].beam_name for x in beam_range]
        self.fx_group = [values[x].fx_group for x in beam_range]
        self.fxs = [values[x].fxs for x in beam_range]
        self.fx_grp_beam_count = [values[x].fx_grp_beam_count for x in beam_range]
        self.beam_dose = [values[x].beam_dose for x in beam_range]
        self.beam_mu = [values[x].beam_mu for x in beam_range]
        self.radiation_type = [values[x].radiation_type for x in beam_range]
        self.beam_energy = [values[x].beam_energy for x in beam_range]
        self.beam_type = [values[x].beam_type for x in beam_range]
        self.control_point_count = [values[x].control_point_count for x in beam_range]
        self.gantry_start = [values[x].gantry_start for x in beam_range]
        self.gantry_end = [values[x].gantry_end for x in beam_range]
        self.gantry_rot_dir = [values[x].gantry_rot_dir for x in beam_range]
        self.collimator_angle = [values[x].collimator_angle for x in beam_range]
        self.couch_angle = [values[x].couch_angle for x in beam_range]
        self.isocenter = [values[x].isocenter for x in beam_range]
        self.ssd = [values[x].ssd for x in beam_range]
        self.treatment_machine = [values[x].treatment_machine for x in beam_range]


class RxTable:
    def __init__(self, plan_file, structure_file):
        values = {}
        rt_plan = dicom.read_file(plan_file)
        rt_structure = dicom.read_file(structure_file)
        fx_grp_seq = rt_plan.FractionGroupSequence

        # Record Medical Record Number
        mrn = rt_plan.PatientID

        # This UID must be in all DICOM files associated with this sim study
        study_instance_uid = rt_plan.StudyInstanceUID

        fx_group_count = len(fx_grp_seq)

        for i in range(0, fx_group_count):
            rx_dose = 0
            fx_grp_number = i + 1
            fx_dose = 0
            fx_grp_name = 'default'
            plan_name = rt_plan.RTPlanLabel

            for j in range(0, fx_grp_seq[i].NumberOfBeams):
                fx_dose += fx_grp_seq[i].ReferencedBeamSequence[j].BeamDose

            fxs = fx_grp_seq[i].NumberOfFractionsPlanned
            rx_percent = float(100)
            normalization_method = 'point'
            normalization_object = 'calc point'

            # Because DICOM does not contain Rx's explicitly, the user must create
            # a point in the RT Structure file called 'rx[#]: ' per rx
            roi_counter = 0
            roi_seq = rt_structure.StructureSetROISequence  # just limit num characters in code
            roi_count = len(roi_seq)
            # set defaults in case rx and tx points are not found

            rx_pt_found = False
            while roi_counter < roi_count and not rx_pt_found:
                roi_name = roi_seq[roi_counter].ROIName.lower()
                if len(roi_name) > 2:
                    temp = roi_name.split(':')

                    if temp[0][0:3] == 'rx ' and int(temp[0].strip('rx ')) == i + 1:
                        fx_grp_number = int(temp[0].strip('rx '))

                        fx_grp_name = temp[1].strip()

                        fx_dose = float(temp[2].split('cgy')[0]) / float(100)
                        fxs = int(temp[2].split('x ')[1].split(' to')[0])
                        rx_dose = fx_dose * float(fxs)

                        rx_percent = float(temp[2].strip().split(' ')[5].strip('%'))
                        normalization_method = temp[3].strip()

                        if normalization_method != 'plan_max':
                            normalization_object = temp[4].strip()
                        else:
                            normalization_object = 'plan_max'
                        rx_pt_found = True

                roi_counter += 1

            current_fx_grp_row = RxRow(mrn, study_instance_uid, plan_name, fx_grp_name, fx_grp_number,
                                       fx_group_count, fx_dose, fxs, rx_dose, rx_percent, normalization_method,
                                       normalization_object)
            values[i] = current_fx_grp_row

        fx_group_range = range(0, fx_group_count)
        # Rearrange values into separate variables
        self.mrn = [values[x].mrn for x in fx_group_range]
        self.study_instance_uid = [values[x].study_instance_uid for x in fx_group_range]
        self.plan_name = [values[x].plan_name for x in fx_group_range]
        self.fx_grp_name = [values[x].fx_grp_name for x in fx_group_range]
        self.fx_grp_number = [values[x].fx_grp_number for x in fx_group_range]
        self.fx_grp_count = [values[x].fx_grp_count for x in fx_group_range]
        self.fx_dose = [values[x].fx_dose for x in fx_group_range]
        self.fxs = [values[x].fxs for x in fx_group_range]
        self.rx_dose = [values[x].rx_dose for x in fx_group_range]
        self.rx_percent = [values[x].rx_percent for x in fx_group_range]
        self.normalization_method = [values[x].normalization_method for x in fx_group_range]
        self.normalization_object = [values[x].normalization_object for x in fx_group_range]
        self.count = fx_group_count


if __name__ == '__main__':
    pass



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
main program for Bokeh server
Created on Sun Apr 21 2017
@author: Dan Cutright, PhD
"""

import numpy as np
import itertools
from bokeh.layouts import layout, column, row
from bokeh.models import ColumnDataSource, Legend, CustomJS, HoverTool
from bokeh.models.widgets import Select, Button, PreText, TableColumn, DataTable, \
    NumberFormatter, RadioButtonGroup, TextInput, RadioGroup
from bokeh.plotting import figure
from bokeh.io import curdoc
from roi_name_manager import DatabaseROIs
from analysis_tools import DVH, get_study_instance_uids
from sql_connector import DVH_SQL
from sql_to_python import QuerySQL
from bokeh.palettes import Category20_9 as palette
from datetime import datetime
from utilities import Temp_DICOM_FileSet
from dicompylercore import dvhcalc, dicomparser
import dicom

# Declare variables
widget_row_origin = 2
db_rois = DatabaseROIs()
colors = itertools.cycle(palette)
query_row = []
query_row_type = []
endpoint_columns = {}
temp_dvh_info = Temp_DICOM_FileSet()
for i in range(0, 10):
    endpoint_columns[i] = ''
dvh_review_mrns = temp_dvh_info.mrn
if dvh_review_mrns[0] != '':
    dvh_review_rois = temp_dvh_info.get_roi_names(dvh_review_mrns[0]).values()
else:
    dvh_review_rois = ['']
x = []
y = []

# Initialize ColumnDataSource variables
temp = []
temp_vec = []
temp_null = []
for i in range(0, 6):
    temp.append(0)
    temp_vec.append([''])
    temp_null.append('')
source = ColumnDataSource(data=dict(x=temp_vec,
                                    y=temp_vec,
                                    roi_name=temp_null,
                                    volume=temp,
                                    min_dose=temp,
                                    mean_dose=temp,
                                    max_dose=temp,
                                    eud=temp,
                                    eud_a_value=temp,
                                    ep1=temp_null,
                                    ep2=temp_null,
                                    ep3=temp_null,
                                    ep4=temp_null,
                                    ep5=temp_null,
                                    ep6=temp_null,
                                    ep7=temp_null,
                                    ep8=temp_null))
source_patch = ColumnDataSource(data=dict(x_patch=[], y_patch=[]))
source_stats = ColumnDataSource(data=dict(x=[], min=[], q1=[], mean=[], median=[], q3=[], max=[]))
source_endpoint_names = ColumnDataSource(data=dict(ep1=[''], ep2=[''], ep3=[''], ep4=[''], ep5=[''],
                                                   ep6=[''], ep7=[''], ep8=['']))
source_endpoints = ColumnDataSource(data=dict())


# Categories map of dropdown values, SQL column, and SQL table (and data source for range_categories)
selector_categories = {'ROI Institutional Category': {'var_name': 'institutional_roi', 'table': 'DVHs'},
                       'ROI Physician Category': {'var_name': 'physician_roi', 'table': 'DVHs'},
                       'ROI Type': {'var_name': 'roi_type', 'table': 'DVHs'},
                       'Beam Energy': {'var_name': 'beam_energy', 'table': 'Beams'},
                       'Beam Type': {'var_name': 'beam_type', 'table': 'Beams'},
                       'Dose Grid Resolution': {'var_name': 'dose_grid_res', 'table': 'Plans'},
                       'Gantry Rotation Direction': {'var_name': 'gantry_rot_dir', 'table': 'Beams'},
                       'Radiation Type': {'var_name': 'radiation_type', 'table': 'Beams'},
                       'Patient Orientation': {'var_name': 'patient_orientation', 'table': 'Plans'},
                       'Patient Sex': {'var_name': 'patient_sex', 'table': 'Plans'},
                       'Physician': {'var_name': 'physician', 'table': 'Plans'},
                       'Tx Modality': {'var_name': 'tx_modality', 'table': 'Plans'},
                       'Tx Site': {'var_name': 'tx_site', 'table': 'Plans'},
                       'Normalization': {'var_name': 'normalization_method', 'table': 'Rxs'},
                       'Treatment Machine': {'var_name': 'treatment_machine', 'table': 'Beams'},
                       'Heterogeneity Correction': {'var_name': 'heterogeneity_correction', 'table': 'Plans'}}
range_categories = {'Age': {'var_name': 'age', 'table': 'Plans', 'units': ''},
                    'Birth Date': {'var_name': 'birth_date', 'table': 'Plans', 'units': ''},
                    'Planned Fractions': {'var_name': 'fxs', 'table': 'Plans', 'units': ''},
                    'Rx Dose': {'var_name': 'rx_dose', 'table': 'Plans', 'units': 'Gy'},
                    'Rx Isodose': {'var_name': 'rx_percent', 'table': 'Rxs', 'units': '%'},
                    'Simulation Date': {'var_name': 'sim_study_date', 'table': 'Plans', 'units': ''},
                    'Total Plan MU': {'var_name': 'total_mu', 'table': 'Plans', 'units': ''},
                    'Fraction Dose': {'var_name': 'fx_dose', 'table': 'Rxs', 'units': 'Gy'},
                    'Beam Dose': {'var_name': 'beam_dose', 'table': 'Beams', 'units': 'Gy'},
                    'Beam MU': {'var_name': 'beam_mu', 'table': 'Beams', 'units': ''},
                    'Control Point Count': {'var_name': 'control_point_count', 'table': 'Beams', 'units': ''},
                    'Collimator Angle': {'var_name': 'collimator_angle', 'table': 'Beams', 'units': 'deg'},
                    'Couch Angle': {'var_name': 'couch_angle', 'table': 'Beams', 'units': 'deg'},
                    'Gantry Start Angle': {'var_name': 'gantry_start', 'table': 'Beams', 'units': 'deg'},
                    'Gantry End Angle': {'var_name': 'gantry_end', 'table': 'Beams', 'units': 'deg'},
                    'SSD': {'var_name': 'ssd', 'table': 'Beams', 'units': 'cm'},
                    'ROI Min Dose': {'var_name': 'min_dose', 'table': 'DVHs', 'units': 'Gy'},
                    'ROI Mean Dose': {'var_name': 'mean_dose', 'table': 'DVHs', 'units': 'Gy'},
                    'ROI Max Dose': {'var_name': 'max_dose', 'table': 'DVHs', 'units': 'Gy'},
                    'ROI Volume': {'var_name': 'volume', 'table': 'DVHs', 'units': 'cc'}}


# Functions that add widget rows
def button_add_selector_row():
    global query_row_type
    old_label = main_add_selector_button.label
    old_type = main_add_selector_button.button_type
    main_add_selector_button.label = 'Updating Layout...'
    main_add_selector_button.button_type = 'warning'
    query_row.append(AddSelectorRow())
    layout.children.insert(widget_row_origin + len(query_row_type), query_row[-1].row)
    query_row_type.append('selector')
    update_update_button_status()
    main_add_selector_button.label = old_label
    main_add_selector_button.button_type = old_type


def button_add_range_row():
    global query_row_type
    old_label = main_add_range_button.label
    old_type = main_add_range_button.button_type
    main_add_range_button.label = 'Updating Layout...'
    main_add_range_button.button_type = 'warning'
    query_row.append(AddRangeRow())
    layout.children.insert(widget_row_origin + len(query_row_type), query_row[-1].row)
    query_row_type.append('range')
    update_update_button_status()
    main_add_range_button.label = old_label
    main_add_range_button.button_type = old_type


# Updates query row ids (after row deletion)
def update_query_row_ids():
    global query_row
    for i in range(1, len(query_row)):
        query_row[i].id = i


# find the widget rows that need text input title updates
# and then call update_range_values to the needed widget row
def update_all_range_endpoints():
    global query_row, query_row_type
    for i in range(1, len(query_row)):
        if query_row_type[i] == 'range':
            query_row[i].update_range_values(query_row[i].select_category.value)


# main update function
def update_data():
    global query_row_type, query_row
    old_update_button_label = update_button.label
    old_update_button_type = update_button.button_type
    update_button.label = 'Updating...'
    update_button.button_type = 'warning'
    uids, dvh_query_str = get_query()
    print str(datetime.now()), 'getting dvh data'
    current_dvhs = DVH(uid=uids, dvh_condition=dvh_query_str)
    print str(datetime.now()), 'initializing source data', current_dvhs.query
    update_dvh_data(current_dvhs)
    update_all_range_endpoints()
    update_button.label = old_update_button_label
    update_button.button_type = old_update_button_type


# Checks size of current query, changes update button to orange if over 50 DVHs
def update_update_button_status():
    uids, dvh_query_str = get_query()
    count = DVH_SQL().get_roi_count_from_query(dvh_condition=dvh_query_str, uid=uids)
    if count > 50:
        update_button.button_type = "warning"
        update_button.label = "Large Update (" + str(count) + ")"
    else:
        update_button.button_type = "success"
        update_button.label = "Update Stats"


# This function retuns the list of information needed to execute QuerySQL from
# SQL_to_Python.py (i.e., uids and dvh_condition
def get_query():
    global query_row_type, query_row
    plan_query_map = {}
    rx_query_map = {}
    beam_query_map = {}
    dvh_query_map = {}
    for i in range(1, len(query_row)):
        if query_row_type[i] in {'selector', 'range'}:
            if query_row_type[i] == 'selector':
                var_name = selector_categories[query_row[i].select_category.value]['var_name']
                table = selector_categories[query_row[i].select_category.value]['table']
                value = query_row[i].select_value.value
                query_str = var_name + " = '" + value + "'"
            elif query_row_type[i] == 'range':
                query_row[i].select_category.value
                var_name = range_categories[query_row[i].select_category.value]['var_name']
                table = range_categories[query_row[i].select_category.value]['table']
                if query_row[i].text_min.value != '':
                    value_low = query_row[i].text_min.value
                else:
                    value_low = query_row[i].min_value
                if query_row[i].text_max.value != '':
                    value_high = query_row[i].text_max.value
                else:
                    value_high = query_row[i].max_value
                query_str = var_name + " between " + str(value_low) + " and " + str(value_high)

            if table == 'Plans':
                if var_name not in plan_query_map:
                    plan_query_map[var_name] = []
                plan_query_map[var_name].append(query_str)
            elif table == 'Rxs':
                if var_name not in rx_query_map:
                    rx_query_map[var_name] = []
                rx_query_map[var_name].append(query_str)
            elif table == 'Beams':
                if var_name not in beam_query_map:
                    beam_query_map[var_name] = []
                beam_query_map[var_name].append(query_str)
            elif table == 'DVHs':
                if var_name not in dvh_query_map:
                    dvh_query_map[var_name] = []
                dvh_query_map[var_name].append(query_str)

    # The multiple entries of the same variable are found, assume an 'or' condition
    plan_query_str = []
    for key in plan_query_map.iterkeys():
        plan_query_str.append('(' + ' or '.join(plan_query_map[key]) + ')')

    rx_query_str = []
    for key in rx_query_map.iterkeys():
        rx_query_str.append('(' + ' or '.join(rx_query_map[key]) + ')')

    beam_query_str = []
    for key in beam_query_map.iterkeys():
        beam_query_str.append('(' + ' or '.join(beam_query_map[key]) + ')')

    dvh_query_str = []
    skip_roi_names = False
    roi_categories = {'physician_roi',
                      'institutional_roi'}
    if 'physician_roi' in dvh_query_map.keys():
        if 'institutional_roi' in dvh_query_map.keys():
            roi_name_query = dvh_query_map['physician_roi'] + \
                             dvh_query_map['institutional_roi']
            dvh_query_str.append('(' + ' or '.join(roi_name_query) + ')')
            skip_roi_names = True
    for key in dvh_query_map.iterkeys():
        if skip_roi_names and key in roi_categories:
            pass
        else:
            dvh_query_str.append('(' + ' or '.join(dvh_query_map[key]) + ')')

    # assumes an 'and' condition between variable types
    plan_query_str = ' and '.join(plan_query_str)
    rx_query_str = ' and '.join(rx_query_str)
    beam_query_str = ' and '.join(beam_query_str)
    dvh_query_str = ' and '.join(dvh_query_str)

    print str(datetime.now()), 'getting uids'
    print str(datetime.now()), 'Plans = ', plan_query_str
    print str(datetime.now()), 'Rxs = ', rx_query_str
    print str(datetime.now()), 'Beams = ', beam_query_str
    uids = get_study_instance_uids(Plans=plan_query_str, Rxs=rx_query_str, Beams=beam_query_str)['union']

    return uids, dvh_query_str


# This class adds a row suitable to filter discrete data from the SQL database
class AddSelectorRow:
    def __init__(self):

        self.id = len(query_row)
        self.category_options = selector_categories.keys()
        self.category_options.sort()
        self.select_category = Select(value="ROI Institutional Category", options=self.category_options, width=450)
        self.select_category.on_change('value', self.update_selector_values)

        self.sql_table = selector_categories[self.select_category.value]['table']
        self.var_name = selector_categories[self.select_category.value]['var_name']
        self.values = DVH_SQL().get_unique_values(self.sql_table, self.var_name)
        self.select_value = Select(value=self.values[0], options=self.values, width=450)
        self.select_value.on_change('value', self.selector_values_ticker)

        self.delete_last_row = Button(label="Delete", button_type="warning", width=100)
        self.delete_last_row.on_click(self.delete_row)

        self.row = row(self.select_category,
                       self.select_value,
                       self.delete_last_row)

    def update_selector_values(self, attrname, old, new):
        table_new = selector_categories[self.select_category.value]['table']
        var_name_new = selector_categories[new]['var_name']
        new_options = DVH_SQL().get_unique_values(table_new, var_name_new)
        self.select_value.options = new_options
        self.select_value.value = new_options[0]
        update_update_button_status()

    def selector_values_ticker(self, attrname, old, new):
        update_update_button_status()

    def delete_row(self):
        del (layout.children[widget_row_origin + self.id])
        query_row_type.pop(self.id)
        query_row.pop(self.id)
        update_query_row_ids()
        update_update_button_status()


# This class adds a row suitable to filter continuous data from the SQL database
class AddRangeRow:
    def __init__(self):
        self.id = len(query_row)
        self.category_options = range_categories.keys()
        self.category_options.sort()
        self.select_category = Select(value=self.category_options[-1], options=self.category_options, width=450)
        self.select_category.on_change('value', self.update_range_values_ticker)

        self.sql_table = range_categories[self.select_category.value]['table']
        self.var_name = range_categories[self.select_category.value]['var_name']
        self.min_value = []
        self.max_value = []
        self.text_min = TextInput(value='', title='', width=225)
        self.text_min.on_change('value', self.check_for_start_date_ticker)
        self.text_max = TextInput(value='', title='', width=225)
        self.text_max.on_change('value', self.check_for_end_date_ticker)
        self.update_range_values(self.select_category.value)

        self.delete_last_row = Button(label="Delete", button_type="warning", width=100)
        self.delete_last_row.on_click(self.delete_row)

        self.row = row([self.select_category,
                        self.text_min,
                        self.text_max,
                        self.delete_last_row])

    def check_for_start_date_ticker(self, attrname, old, new):
        if self.select_category.value.lower().find("date") > -1:
            self.text_min.value = date_str_to_SQL_format(new, type='start')

    def check_for_end_date_ticker(self, attrname, old, new):
        if self.select_category.value.lower().find("date") > -1:
            self.text_max.value = date_str_to_SQL_format(new, type='end')

    def update_range_values_ticker(self, attrname, old, new):
        self.update_range_values(new)

    def update_range_values(self, new):
        table_new = range_categories[new]['table']
        var_name_new = range_categories[new]['var_name']
        self.min_value = DVH_SQL().get_min_value(table_new, var_name_new)
        self.max_value = DVH_SQL().get_max_value(table_new, var_name_new)
        self.text_min.title = 'Min: ' + str(self.min_value) + ' ' + range_categories[new]['units']
        self.text_max.title = 'Max: ' + str(self.max_value) + ' ' + range_categories[new]['units']
        update_update_button_status()

    def delete_row(self):
        del (layout.children[widget_row_origin + self.id])
        query_row_type.pop(self.id)
        query_row.pop(self.id)
        update_query_row_ids()
        update_update_button_status()


# This class adds a row to calculate specified DVH endpoints
class AddEndPointRow:
    def __init__(self, instance):

        self.id = len(query_row)
        self.options = ["Dose to Vol", "Vol of Dose"]
        self.select_category = RadioButtonGroup(labels=self.options, active=0, width=225)
        self.select_category.on_change('active', self.endpoint_category_ticker)

        self.unit_labels = [["%", "cc"], ["%", "Gy"]]

        self.units_out = RadioButtonGroup(labels=self.unit_labels[1], active=1, width=225)
        self.units_out.on_change('active', self.endpoint_units_out_ticker)

        self.text_input = TextInput(value='', title="Volume (%):", width=300)
        self.text_input.on_change('value', self.endpoint_calc_ticker)

        self.units = RadioButtonGroup(labels=self.unit_labels[0], active=0, width=225)
        self.units.on_change('active', self.endpoint_units_ticker)

        self.label = PreText(text='Column ' + str(instance), width=100)

        self.row = row(self.label,
                       self.select_category,
                       self.units_out,
                       self.text_input,
                       self.units)

    def endpoint_category_ticker(self, attrname, old, new):
        self.update_text_input_title()
        self.units.labels = self.unit_labels[new]
        self.units_out.labels = self.unit_labels[old]
        if self.text_input.value != '':
            update_endpoint_data()

    def endpoint_calc_ticker(self, attrname, old, new):
        if self.text_input.value != '':
            update_endpoint_data()

    def endpoint_units_ticker(self, attrname, old, new):
        self.update_text_input_title()
        if self.text_input.value != '':
            update_endpoint_data()

    def endpoint_units_out_ticker(self, attrname, old, new):
        if self.text_input.value != '':
            update_endpoint_data()

    def update_text_input_title(self):
        if self.select_category.active == 0:
            self.text_input.title = 'Volume (' + self.unit_labels[0][self.units.active] + '):'
        elif self.select_category.active == 1:
            self.text_input.title = 'Dose (' + self.unit_labels[1][self.units.active] + '):'


# input is a DVH class from Analysis_Tools.py
# This function creates a new ColumnSourceData and calls
# the functions to update beam, rx, and plans ColumnSourceData variables
def update_dvh_data(dvh):

    update_button.label = 'Getting DVH data...'
    print str(datetime.now()), 'updating dvh data'
    line_colors = []
    for i, color in itertools.izip(range(0, dvh.count), colors):
        line_colors.append(color)

    x_axis = np.add(np.linspace(0, dvh.bin_count, dvh.bin_count) / float(100), 0.005)

    x_data = []
    y_data = []
    endpoint_columns = []
    x_scale = []
    y_scale = []
    for i in range(0, dvh.count):
        endpoint_columns.append('')
        x_data.append(x_axis.tolist())
        y_data.append(dvh.dvh[:, i].tolist())

    print str(datetime.now()), 'beginning stat calcs'
    update_button.label = 'Calculating stats...'

    print str(datetime.now()), 'calculating patches'
    stat_dvhs = dvh.get_standard_stat_dvh(dose='absolute', volume='relative')

    print str(datetime.now()), 'patches calculated'

    source_patch.data = {'x_patch': np.append(x_axis, x_axis[::-1]).tolist(),
                         'y_patch': np.append(stat_dvhs['q3'], stat_dvhs['q1'][::-1]).tolist()}
    print str(datetime.now()), 'patches set'
    source_stats.data = {'x': x_axis.tolist(),
                         'min': stat_dvhs['min'].tolist(),
                         'q1': stat_dvhs['q1'].tolist(),
                         'mean': stat_dvhs['mean'].tolist(),
                         'median': stat_dvhs['median'].tolist(),
                         'q3': stat_dvhs['q3'].tolist(),
                         'max': stat_dvhs['max'].tolist()}

    print str(datetime.now()), 'stats set'

    update_beam_data(dvh.study_instance_uid)
    update_plan_data(dvh.study_instance_uid)
    update_rx_data(dvh.study_instance_uid)


# updates beam ColumnSourceData for a given list of uids
def update_beam_data(uids):
    cond_str = "study_instance_uid in ('" + "', '".join(uids) + "')"
    beam_data = QuerySQL('Beams', cond_str)


# updates plan ColumnSourceData for a given list of uids
def update_plan_data(uids):

    cond_str = "study_instance_uid in ('" + "', '".join(uids) + "')"
    plan_data = QuerySQL('Plans', cond_str)


# updates rx ColumnSourceData for a given list of uids
def update_rx_data(uids):

    cond_str = "study_instance_uid in ('" + "', '".join(uids) + "')"
    rx_data = QuerySQL('Rxs', cond_str)


# updates endpoint ColumnSourceData for a given DVH class from Analysis_Tools.py
# note that endpoint ColumnSourceData exits in dvh data ColumnSourceData (i.e.,
# the main ColumnSourceData variable, or 'source')
def update_endpoint_data():
    global query_row, query_row_type, endpoint_columns
    for i in range(0, 8):
        endpoint_columns[i] = ''
    endpoints_map = {}
    counter = 0
    for i in range(0, len(query_row)):
        if query_row_type[i] == 'endpoint' and counter < 8:
            output_unit = ['Gy', 'cc']
            x = float(query_row[i].text_input.value)
            x_for_text = x
            if query_row[i].units.active == 0:
                endpoint_input = 'relative'
                units = query_row[i].units.labels[0]
                x /= 100
            else:
                endpoint_input = 'absolute'
                units = query_row[i].units.labels[1]
            if query_row[i].units_out.active == 0:
                endpoint_output = 'relative'
                output_unit = ['%', '%']
            else:
                endpoint_output = 'absolute'
            if query_row[i].select_category.active == 0:
                endpoint_columns[counter] = 'D_' + str(x_for_text) + units + ' (' + output_unit[0] + ')'
                endpoints_map[counter] = dvh.get_dose_to_volume(x, input=endpoint_input, output=endpoint_output)
            else:
                endpoint_columns[counter] = 'V_' + str(x_for_text) + units + ' (' + output_unit[1] + ')'
                endpoints_map[counter] = dvh.get_volume_of_dose(x, input=endpoint_input, output=endpoint_output)
            counter += 1
    for i in range(counter, 8):
        endpoints_map[i] = []
        for j in range(0, 1):
            endpoints_map[i].append('')

    tuple_list = {}
    for i in range(0, 8):
        current_tuple_list = []
        for j in range(0, 1):
            current_tuple_list.append(tuple([j, endpoints_map[i][j]]))
        tuple_list[i] = current_tuple_list

    patches = {'ep1': tuple_list[0],
               'ep2': tuple_list[1],
               'ep3': tuple_list[2],
               'ep4': tuple_list[3],
               'ep5': tuple_list[4],
               'ep6': tuple_list[5],
               'ep7': tuple_list[6],
               'ep8': tuple_list[7]}

    source.patch(patches)

    source_endpoint_names.data = {'ep1': [endpoint_columns[0]],
                                  'ep2': [endpoint_columns[1]],
                                  'ep3': [endpoint_columns[2]],
                                  'ep4': [endpoint_columns[3]],
                                  'ep5': [endpoint_columns[4]],
                                  'ep6': [endpoint_columns[5]],
                                  'ep7': [endpoint_columns[6]],
                                  'ep8': [endpoint_columns[7]]}

    # Update table column names based on the query
    for i in range(0, counter):
        data_table_endpoints.columns[i+1] = TableColumn(field=str('ep' + str(i+1)),
                                                        title=endpoint_columns[i],
                                                        width=100,
                                                        formatter=NumberFormatter(format="0.00"))


def update_dvh_review_mrns():
    global temp_dvh_info, dvh_review_mrns
    temp_dvh_info = Temp_DICOM_FileSet()
    dvh_review_mrns = temp_dvh_info.mrn
    select_reviewed_mrn.options = dvh_review_mrns


def update_dvh_review_rois(attr, old, new):
    global temp_dvh_info, dvh_review_rois
    initial_button_type = calculate_review_dvh_button.button_type
    calculate_review_dvh_button.button_type = "warning"
    initial_label = calculate_review_dvh_button.label
    calculate_review_dvh_button.label = "Updating..."
    if new != '':
        dvh_review_rois = temp_dvh_info.get_roi_names(new).values()
        select_reviewed_dvh.options = dvh_review_rois
        select_reviewed_dvh.value = dvh_review_rois[0]
    else:
        select_reviewed_dvh.options = ['']
        select_reviewed_dvh.value = ['']

    calculate_review_dvh_button.button_type = initial_button_type
    calculate_review_dvh_button.label = initial_label


def calculate_review_dvh():
    global temp_dvh_info, dvh_review_rois, x, y

    initial_button_type = calculate_review_dvh_button.button_type
    initial_button_label = calculate_review_dvh_button.label
    calculate_review_dvh_button.button_type = 'warning'
    calculate_review_dvh_button.label = 'Calculating...'

    file_index = temp_dvh_info.mrn.index(select_reviewed_mrn.value)
    roi_index = dvh_review_rois.index(select_reviewed_dvh.value)
    structure_file = temp_dvh_info.structure[file_index]
    dose_file = temp_dvh_info.dose[file_index]
    key = temp_dvh_info.get_roi_names(select_reviewed_mrn.value).keys()[roi_index]

    rt_st = dicomparser.DicomParser(structure_file)
    rt_structures = rt_st.GetStructures()
    review_dvh = dvhcalc.get_dvh(structure_file, dose_file, key)

    roi_name = rt_structures[key]['name']
    volume = review_dvh.volume
    min_dose = review_dvh.min
    mean_dose = review_dvh.mean
    max_dose = review_dvh.max
    eud = 0
    eud_a_value = 0
    x = review_dvh.bincenters
    y = np.divide(review_dvh.counts, max(review_dvh.counts)).tolist()

    patches = {'x': [(0, x)],
               'y': [(0, y)],
               'roi_name': [(0, roi_name)],
               'volume': [(0, volume)],
               'min_dose': [(0, min_dose)],
               'mean_dose': [(0, mean_dose)],
               'max_dose': [(0, max_dose)],
               'eud': [(0, eud)],
               'eud_a_value': [(0, eud_a_value)]}

    source.patch(patches)

    calculate_review_dvh_button.button_type = initial_button_type
    calculate_review_dvh_button.label = initial_button_label


def date_str_to_SQL_format(date, **kwargs):

    if kwargs['type'] == 'start':
        append_month = '-01-01'
        append_day = '-01'
    else:
        append_month = '-12-31'
        append_day = '-31'
    date_len = len(date)
    if date.isdigit():
        if date_len == 4:
            date = date + append_month
        elif date_len == 5:
            date = date[0:4] + '0' + date[4] + append_day
        elif date_len == 6:
            date = date + append_day
    else:
        non_digit_count = date_len - sum(x.isdigit() for x in date)
        if non_digit_count == 1:
            date = date[0:4] + date[5:date_len]
        else:
            date = date[0:4] + date[5:7] + date[8:date_len]

    return date


# set up layout
tools = "pan,wheel_zoom,box_zoom,reset,crosshair,save,hover"
dvh_plots = figure(plot_width=1000, plot_height=400, tools=tools, logo=None, active_drag="box_zoom")

# Add statistical plots to figure
stats_min = dvh_plots.line('x', 'min',
                           source=source_stats,
                           line_width=2,
                           color='black',
                           line_dash='dotted',
                           alpha=0.2)
stats_q1 = dvh_plots.line('x', 'q1',
                          source=source_stats,
                          line_width=0.5,
                          color='black',
                          alpha=0.2)
stats_median = dvh_plots.line('x', 'median',
                              source=source_stats,
                              line_width=2,
                              color='lightpink',
                              line_dash='dashed',
                              alpha=0.75)
stats_mean = dvh_plots.line('x', 'mean',
                            source=source_stats,
                            line_width=2,
                            color='lightseagreen',
                            line_dash='dashed',
                            alpha=0.5)
stats_q3 = dvh_plots.line('x', 'q3',
                          source=source_stats,
                          line_width=0.5,
                          color='black',
                          alpha=0.2)
stats_max = dvh_plots.line('x', 'max',
                           source=source_stats,
                           line_width=2,
                           color='black',
                           line_dash='dotted',
                           alpha=0.2)

# Add all DVHs, but hide them until selected
dvh_plots.multi_line('x', 'y', source=source, line_width=2)

# Shaded region between Q1 and Q3
dvh_plots.patch('x_patch', 'y_patch',
                source=source_patch,
                alpha=0.15)

# Set x and y axis labels
dvh_plots.xaxis.axis_label = "Dose (Gy)"
dvh_plots.yaxis.axis_label = "Normalized Volume"

# Set the legend (for stat dvhs only)
legend_stats = Legend(items=[
        ("Max", [stats_max]),
        ("Q3", [stats_q3]),
        ("Median", [stats_median]),
        ("Mean", [stats_mean]),
        ("Q1", [stats_q1]),
        ("Min", [stats_min])
    ], location=(0, 30))

# Add the layout outside the plot, clicking legend item hides the line
dvh_plots.add_layout(legend_stats, 'right')
dvh_plots.legend.click_policy = "hide"

# Set up DataTable for dvhs
data_table_title = PreText(text="DVHs", width=1000)
columns = [TableColumn(field="roi_name", title="ROI Name"),
           TableColumn(field="volume", title="Volume", width=80, formatter=NumberFormatter(format="0.00")),
           TableColumn(field="min_dose", title="Min Dose", width=80, formatter=NumberFormatter(format="0.00")),
           TableColumn(field="mean_dose", title="Mean Dose", width=80, formatter=NumberFormatter(format="0.00")),
           TableColumn(field="max_dose", title="Max Dose", width=80, formatter=NumberFormatter(format="0.00")),
           TableColumn(field="eud", title="EUD", width=80, formatter=NumberFormatter(format="0.00")),
           TableColumn(field="eud_a_value", title="a", width=80)]
data_table = DataTable(source=source, columns=columns, width=1000, height=60)

# Set up EndPoint DataTable
endpoint_table_title = PreText(text="DVH Endpoints", width=1000)
columns = [TableColumn(field="ep1", title=endpoint_columns[0], width=120),
           TableColumn(field="ep2", title=endpoint_columns[1], width=120),
           TableColumn(field="ep3", title=endpoint_columns[2], width=120),
           TableColumn(field="ep4", title=endpoint_columns[3], width=120),
           TableColumn(field="ep5", title=endpoint_columns[4], width=120),
           TableColumn(field="ep6", title=endpoint_columns[5], width=120),
           TableColumn(field="ep7", title=endpoint_columns[6], width=120),
           TableColumn(field="ep8", title=endpoint_columns[7], width=120)]
data_table_endpoints = DataTable(source=source, columns=columns, width=1000, height=75)

# Setup selectors for dvh review
select_reviewed_mrn = Select(title='MRN to review',
                             value=dvh_review_mrns[0],
                             options=dvh_review_mrns,
                             width=200)
select_reviewed_mrn.on_change('value', update_dvh_review_rois)

select_reviewed_dvh = Select(title='ROI',
                             value=dvh_review_rois[0],
                             options=dvh_review_rois,
                             width=500)

calculate_review_dvh_button = Button(label="Calculate Review DVH",
                                     button_type="success",
                                     width=200)
calculate_review_dvh_button.on_click(calculate_review_dvh)

update_dvh_review_mrns()

# Begin defining main row of widgets below figure

# define Update button
update_button = Button(label="Update Stats", button_type="success", width=200)
update_button.on_click(update_data)

# define button to widget row for discrete data filtering
main_add_selector_button = Button(label="Add Selection Filter", button_type="primary", width=200)
main_add_selector_button.on_click(button_add_selector_row)

# define button to widget row for continuous data filtering
main_add_range_button = Button(label="Add Range Filter", button_type="primary", width=200)
main_add_range_button.on_click(button_add_range_row)

# not for display, but add these buttons to query_row
# query row is simply used to keep track of pointers to query rows
# allows code to dynamically add widgets and keep button functionality contained
query_row.append(row(update_button, main_add_selector_button, main_add_range_button))
query_row_type.append('main')

# define main layout to pass to curdoc()
layout = column(dvh_plots,
                row(select_reviewed_mrn, select_reviewed_dvh),
                row(main_add_selector_button,
                    main_add_range_button,
                    update_button,
                    calculate_review_dvh_button),
                data_table_title,
                data_table,
                endpoint_table_title,
                data_table_endpoints,
                AddEndPointRow(1).row,
                AddEndPointRow(2).row,
                AddEndPointRow(3).row,
                AddEndPointRow(4).row,
                AddEndPointRow(5).row,
                AddEndPointRow(6).row,
                AddEndPointRow(7).row,
                AddEndPointRow(8).row,)

# go ahead and add a selector row for the user
button_add_selector_row()

# Create the document Bokeh server will use to generate the webpage
curdoc().add_root(layout)
curdoc().title = "Live Free or DICOM"



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu Mar  2 22:15:52 2017

@author: nightowl
"""

from sql_connector import DVH_SQL
from dicom_to_python import DVHTable, PlanRow, BeamTable, RxTable
import os
import dicom


class DICOM_FileSet:
    def __init__(self, plan, structure, dose):
        self.plan = plan
        self.structure = structure
        self.dose = dose


def dicom_to_sql(**kwargs):
    # Read SQL configuration file
    script_dir = os.path.dirname(__file__)
    rel_path = "preferences/import_settings.txt"
    abs_file_path = os.path.join(script_dir, rel_path)
    with open(abs_file_path, 'r') as document:
        import_settings = {}
        for line in document:
            line = line.split()
            if not line:
                continue
            import_settings[line[0]] = line[1:][0]
            # Convert strings to boolean
            if line[1:][0].lower() == 'true':
                import_settings[line[0]] = True
            elif line[1:][0].lower() == 'false':
                import_settings[line[0]] = False

    if 'start_path' in kwargs and kwargs['start_path']:
        rel_path = kwargs['start_path']
        abs_file_path = os.path.join(script_dir, rel_path)
        import_settings['inbox'] = abs_file_path

    sqlcnx = DVH_SQL()
    force_update = False
    if 'force_update' in kwargs and kwargs['force_update']:
        force_update = True
    f = get_file_paths(import_settings['inbox'], force_update=force_update)

    for n in range(0, len(f)):
        print f[n].structure
        print f[n].plan
        print f[n].dose

        plan = PlanRow(f[n].plan, f[n].structure, f[n].dose)
        beams = BeamTable(f[n].plan)
        dvhs = DVHTable(f[n].structure, f[n].dose)
        rxs = RxTable(f[n].plan, f[n].structure)

        setattr(dvhs, 'ptv_number', rank_ptvs_by_D95(dvhs))

        sqlcnx.insert_plan(plan)
        sqlcnx.insert_beams(beams)
        sqlcnx.insert_dvhs(dvhs)
        sqlcnx.insert_rxs(rxs)

        # Default behavior is to move files from inbox to imported
        # Only way to prevent moving files is to set move_files = False in kwargs
        if 'move_files' not in kwargs:
            move_files_to_imported_path(f[n], import_settings['imported'])
        elif kwargs['move_files']:
            move_files_to_imported_path(f[n], import_settings['imported'])

        # Default behavior is to organize files in the imported folder
        # Only way to prevent organizing files is to set organize_files = False in kwargs
        if 'organize_files' not in kwargs:
            organize_dicom_files(import_settings['imported'])
        elif kwargs['organize_files']:
            organize_dicom_files(import_settings['imported'])

    sqlcnx.cnx.close()


def get_file_paths(start_path, **kwargs):

    f = []
    for root, dirs, files in os.walk(start_path, topdown=False):
        for name in files:
            f.append(os.path.join(root, name))

    plan_files = []
    study_uid_plan = []
    structure_files = []
    study_uid_structure = []
    dose_files = []
    study_uid_dose = []

    if 'force_update' in kwargs and kwargs['force_update']:
        print 'WARNING: Forcing all dicom to be imported (i.e., not checking DB for potential duplicates)'

    db_entry_found = False
    for x in range(0, len(f)):
        if 'force_update' in kwargs and kwargs['force_update']:
            try:
                dicom_file = dicom.read_file(f[x])
                if dicom_file.Modality.lower() == 'rtplan':
                    plan_files.append(f[x])
                    study_uid_plan.append(dicom_file.StudyInstanceUID)
                elif dicom_file.Modality.lower() == 'rtstruct':
                    structure_files.append(f[x])
                    study_uid_structure.append(dicom_file.StudyInstanceUID)
                elif dicom_file.Modality.lower() == 'rtdose':
                    dose_files.append(f[x])
                    study_uid_dose.append(dicom_file.StudyInstanceUID)
            except Exception:
                pass
        else:
            try:
                if not is_file_imported(f[x]):
                    dicom_file = dicom.read_file(f[x])
                    if dicom_file.Modality.lower() == 'rtplan':
                        plan_files.append(f[x])
                        study_uid_plan.append(dicom_file.StudyInstanceUID)
                    elif dicom_file.Modality.lower() == 'rtstruct':
                        structure_files.append(f[x])
                        study_uid_structure.append(dicom_file.StudyInstanceUID)
                    elif dicom_file.Modality.lower() == 'rtdose':
                        dose_files.append(f[x])
                        study_uid_dose.append(dicom_file.StudyInstanceUID)
                else:
                    if not db_entry_found:
                        print "The following files are already imported. Must delete from database before reimporting."
                        db_entry_found = True
                    print f[x]
            except Exception:
                pass

    dicom_files = {}
    for a in range(0, len(plan_files)):
        rt_plan = plan_files[a]
        for b in range(0, len(structure_files)):
            if study_uid_plan[a] == study_uid_structure[b]:
                rt_structure = structure_files[b]
        for c in range(0, len(dose_files)):
            if study_uid_plan[a] == study_uid_dose[c]:
                rt_dose = dose_files[c]
        dicom_files[a] = DICOM_FileSet(rt_plan, rt_structure, rt_dose)

    return dicom_files


def rebuild_database(start_path):
    print 'connecting to SQL DB'
    sqlcnx = DVH_SQL()
    print 'connection established'

    sqlcnx.reinitialize_database()
    print 'DB reinitialized with no data'
    dicom_to_sql(start_path=start_path,
                 force_update=True,
                 move_files=False,
                 organize_files=False)
    sqlcnx.cnx.close()


def is_file_imported(file_path):

    try:
        dicom_data = dicom.read_file(file_path)
        uid = dicom_data.StudyInstanceUID
    except Exception:
        return False

    if dicom_data.Modality.lower() == 'rtplan':
        table_name = 'Plans'
    elif dicom_data.Modality.lower() == 'rtstruct':
        table_name = 'DVHs'
    elif dicom_data.Modality.lower() == 'rtdose':
        table_name = 'DVHs'
    else:
        return False

    if DVH_SQL().is_study_instance_uid_in_table(table_name, uid):
        return True
    else:
        return False


def rank_ptvs_by_D95(dvhs):
    ptv_number_list = []
    ptv_index = []

    for i in range(0, dvhs.count):
        ptv_number_list.append(0)
        if dvhs.roi_type[i] == 'PTV':
            ptv_index.append(i)

    ptv_count = len(ptv_index)

    # Calculate D95 for each PTV
    doses_to_rank = get_dose_to_volume(dvhs, ptv_index, 0.95)
    order_index = sorted(range(ptv_count), key=lambda k: doses_to_rank[k])
    final_order = sorted(range(ptv_count), key=lambda k: order_index[k])

    for i in range(0, ptv_count):
        ptv_number_list[ptv_index[i]] = final_order[i] + 1

    return ptv_number_list


def get_dose_to_volume(dvhs, indices, roi_fraction):
    # Not precise (i.e., no interpolation) but good enough for sorting PTVs
    doses = []
    for x in indices:
        abs_volume = dvhs.volume[x] * roi_fraction
        dvh = dvhs.dvhs[x]
        dose = next(x[0] for x in enumerate(dvh) if x[1] < abs_volume)
        doses.append(dose)

    return doses


def move_files_to_imported_path(file_paths, imported_path):
    for file_type in file_paths.__dict__:
        file_path = getattr(file_paths, file_type)
        file_name = file_path.split('/')[-1]
        new = '/'.join([imported_path, file_name])
        try:
            os.rename(file_path, new)
        except:
            os.mkdir(imported_path)
            os.rename(file_path, new)


def organize_dicom_files(path):
    initial_path = os.path.dirname(os.path.realpath(__file__))

    os.chdir(path)

    files = [f for f in os.listdir('.') if os.path.isfile(os.path.join('.', f))]

    for i in range(0, len(files)):
        try:
            name = dicom.read_file(files[i]).PatientName.replace(' ', '_').replace('^', '_')
            os.mkdir(name)
        except:
            pass

        file_name = files[i].split('/')[-1]

        old = files[i]
        new = '/'.join([path, name, file_name])
        print old, new
        os.rename(old, new)

    os.chdir(initial_path)


if __name__ == '__main__':
    DVH_SQL().reinitialize_database()
    dicom_to_sql()
